@@ -272,12 +272,13 @@ mares_darwin_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t
 
 			// Deco stop
 			if (deco) {
-				sample.event.type = SAMPLE_EVENT_DECOSTOP;
-				sample.event.time = 0;
-				sample.event.flags = 0;
-				sample.event.value = 0;
-				if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+				sample.deco.type = DC_DECO_DECOSTOP;
+			} else {
+				sample.deco.type = DC_DECO_NDL;
 			}
+			sample.deco.time = 0;
+			sample.deco.depth = 0.0;
+			if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
 
 			if (parser->samplesize == 3) {
 				unsigned int type = (time / 20 + 2) % 3;
