@@ -517,7 +517,10 @@ oceanic_vtpro_device_close (dc_device_t *abstract)
 	dc_status_t rc = DC_STATUS_SUCCESS;
 
 	// Switch the device back to surface mode.
-	oceanic_vtpro_quit (device);
+	rc = oceanic_vtpro_quit (device);
+	if (rc != DC_STATUS_SUCCESS) {
+		dc_status_set_error(&status, rc);
+	}
 
 	// Close the device.
 	rc = dc_serial_close (device->port);
