@@ -582,7 +582,7 @@ dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
 
 	if (device == NULL) {
 		status = DC_STATUS_INVALIDARGS;
-		goto out;
+		goto out_invalidargs;
 	}
 
 	RETURN_IF_CUSTOM_SERIAL(device->context,
@@ -667,6 +667,7 @@ dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
 out:
 	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Read", (unsigned char *) data, nbytes);
 
+out_invalidargs:
 	if (actual)
 		*actual = nbytes;
 
@@ -681,7 +682,7 @@ dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *act
 
 	if (device == NULL) {
 		status = DC_STATUS_INVALIDARGS;
-		goto out;
+		goto out_invalidargs;
 	}
 
 	RETURN_IF_CUSTOM_SERIAL(device->context,
@@ -782,6 +783,7 @@ dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *act
 out:
 	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Write", (unsigned char *) data, nbytes);
 
+out_invalidargs:
 	if (actual)
 		*actual = nbytes;
 
