@@ -425,7 +425,7 @@ dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
 
 	if (device == NULL) {
 		status = DC_STATUS_INVALIDARGS;
-		goto out;
+		goto out_invalidargs;
 	}
 
 	RETURN_IF_CUSTOM_SERIAL(device->context,
@@ -450,6 +450,7 @@ dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
 out:
 	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Read", (unsigned char *) data, dwRead);
 
+out_invalidargs:
 	if (actual)
 		*actual = dwRead;
 
@@ -465,7 +466,7 @@ dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *act
 
 	if (device == NULL) {
 		status = DC_STATUS_INVALIDARGS;
-		goto out;
+		goto out_invalidargs;
 	}
 
 	RETURN_IF_CUSTOM_SERIAL(device->context,
@@ -529,6 +530,7 @@ dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *act
 out:
 	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Write", (unsigned char *) data, dwWritten);
 
+out_invalidargs:
 	if (actual)
 		*actual = dwWritten;
 
